@@ -20,7 +20,7 @@
  #define LIBDEFS_H
  #include <libdefs.h>
 #endif
-
+#include <bzero.h>
 #include <swap.h>
 #include <lcrypt.h>
 #include <blowfish.h>
@@ -30,15 +30,26 @@
 #include <safer.h>
 #include <cast.h>
 #include <tean.h>
+#include <twofish/aes.h>
 
 #define MAX_THREADS 100 /* After that pointers are being reused */
 
 /* I think now is re-entrant */
-/* No the init_xxx functions are NOT */
 
+#ifdef HAVE_LIBPTHREAD
+# include <pthread.h>
 static int cur_thread=-1;
+pthread_mutex_t counter_cur_thread = PTHREAD_MUTEX_INITIALIZER;
+#else
+static int cur_thread=-1; /* No threads */
+#endif
+
 static struct cast_key* ckey=NULL; /* For cast */
 static blf_ctx *c=NULL; /* For Blowfish */
+
+static cipherInstance *CI=NULL; /* For Twofish */
+static keyInstance *KI=NULL;    /* For Twofish */
+
 static char **kn1=NULL; /* For DES */
 static char **kn2=NULL;
 static char **kn3=NULL; /* For tripleDES */
@@ -57,7 +68,7 @@ static char **enc_s_register;
 static char **enc_sd_register;
 
 /* hold the key and the algorithm */
-static char **keyword_given=NULL;
+static unsigned char **keyword_given=NULL;
 static int *algorithm_given=NULL;
 
 int get_block_size(int algorithm)
@@ -75,6 +86,10 @@ int get_block_size(int algorithm)
       return 8;
       break;
 
+   case TWOFISH:
+      return 16;
+      break;
+      
    case ThreeWAY:
       return 12;
       break;
@@ -107,6 +122,10 @@ int get_key_size(int algorithm) /* In bytes */
       return 32;
       break;
 
+   case TWOFISH:
+      return 32;
+      break;
+
    case ThreeWAY:
       return 12;
       break;
@@ -124,22 +143,28 @@ int get_key_size(int algorithm) /* In bytes */
 }
 
 
-int init_mcrypt(const int algorithm, void * key,const int lenofkey)
+int init_mcrypt(const int algorithm, void * key, int lenofkey)
 {
 
 char *keyword1, *keyword2, *keyword3; /* For 3DES and safer*/
-char *skey=NULL;
+unsigned char *skey=NULL;
+char twokey[68];
+int i,j;
 
   saferkey=realloc(saferkey, (cur_thread+1)*sizeof(char*));
   kn1=realloc(kn1, (cur_thread+1)*sizeof(char*));
   kn2=realloc(kn2, (cur_thread+1)*sizeof(char*));
   kn3=realloc(kn3, (cur_thread+1)*sizeof(char*));
   c=realloc(c, (cur_thread+1) * sizeof(blf_ctx));
+  CI=realloc(CI, (cur_thread+1) * sizeof(cipherInstance));  
+  KI=realloc(KI, (cur_thread+1) * sizeof(keyInstance));    
   ckey=realloc(ckey, (cur_thread+1) * sizeof(struct cast_key));  
   keyword_given=realloc(keyword_given, (cur_thread+1)*sizeof(char*));
   algorithm_given=realloc(algorithm_given, (cur_thread+1)*sizeof(int));
 
 
+  if (get_key_size(algorithm) < lenofkey) lenofkey=get_key_size(algorithm);
+  
   algorithm_given[cur_thread]=algorithm;
   keyword_given[cur_thread]=calloc(1, get_key_size(algorithm));
   memmove(keyword_given[cur_thread], key, lenofkey);
@@ -152,7 +177,7 @@ char *skey=NULL;
       desinit(0);
       kn1[cur_thread]=malloc(8*16*sizeof(char));
       
-      des_setkey( (void*) kn1[cur_thread], keyword_given[cur_thread]);
+      des_setkey( (void*) kn1[cur_thread], (void *) keyword_given[cur_thread]);
       return cur_thread;
       break;
 
@@ -211,14 +236,34 @@ char *skey=NULL;
        
    case BLOWFISH:
       if (lenofkey<2) {
-          blf_key(&c[cur_thread],keyword_given[cur_thread],2);
+          blf_key(&c[cur_thread], (void*) keyword_given[cur_thread],2);
       }
       else {
-          blf_key(&c[cur_thread],keyword_given[cur_thread],lenofkey);
+          blf_key(&c[cur_thread], (void*) keyword_given[cur_thread],lenofkey);
       }
       return cur_thread;
       break;
 
+   case TWOFISH:
+
+      Bzero( &KI[cur_thread], sizeof(keyInstance));
+      Bzero( &CI[cur_thread], sizeof(cipherInstance));      
+      Bzero( twokey, sizeof(twokey));
+      
+      cipherInit( &CI[cur_thread], MODE_ECB, NULL);
+
+      j=0;
+      for (i=0;i<lenofkey;i++) {
+        sprintf( &twokey[j], "%.2x", (unsigned char) keyword_given[cur_thread][i]);
+        j+=2;
+      }
+      
+        j=makeKey ( &KI[cur_thread], DIR_ENCRYPT, lenofkey*8,  twokey);
+        if (j<0) { return (-1); } 
+
+      return cur_thread;
+      break;
+
    case GOST:
       kboxinit();
       return cur_thread;
@@ -274,6 +319,11 @@ int mcrypt(int thread, void *plaintext)
       return 0;
       break;
 
+   case TWOFISH:
+      blockEncrypt(&CI[thread], &KI[thread], plaintext, 16*8, plaintext); /* 16 bytes */ 
+      return 0;
+      break;
+
    case GOST:
       gostcrypt(plaintext, (void*) keyword_given[thread]);
       return 0;
@@ -299,7 +349,6 @@ int mcrypt(int thread, void *plaintext)
 /* plaintext should be in block's size */
 int mdecrypt(int thread, void *plaintext)
 {
-
   switch(algorithm_given[thread]) {
   
    case SAFER64:
@@ -330,6 +379,11 @@ int mdecrypt(int thread, void *plaintext)
       return 0;
       break;
 
+   case TWOFISH:
+      blockDecrypt(&CI[thread], &KI[thread], plaintext, 16*8, plaintext); /* 16 bytes */
+      return 0;
+      break;
+
    case BLOWFISH:
       deblf(&c[thread], plaintext);
       return 0;
@@ -395,9 +449,15 @@ return 0;
 
 /* ECB MODE */
 
-int init_mcrypt_ecb(const int algorithm, void * key,const int lenofkey)
+int init_mcrypt_ecb(const int algorithm, void * key, int lenofkey)
 {
 
+int x;
+
+#ifdef HAVE_LIBPTHREAD
+pthread_mutex_lock(&counter_cur_thread);
+#endif 
+
 cur_thread++;
 
   if (cur_thread < MAX_THREADS && finished_threads==0) {
@@ -411,10 +471,29 @@ cur_thread++;
        cur_thread++;
      }  
   
-     if (cur_thread >= MAX_THREADS) return -1;
+     if (cur_thread >= MAX_THREADS) {
+        #ifdef HAVE_LIBPTHREAD
+        pthread_mutex_unlock(&counter_cur_thread);
+        #endif
+        return -1;
+     }
   }
 
-return init_mcrypt(algorithm, key, lenofkey);
+x=init_mcrypt(algorithm, key, lenofkey);
+
+ if (x<0) {
+   end_mcrypt(cur_thread);
+   #ifdef HAVE_LIBPTHREAD
+   pthread_mutex_unlock(&counter_cur_thread);
+   #endif
+   return -1;
+ }
+ else {
+   #ifdef HAVE_LIBPTHREAD
+   pthread_mutex_unlock(&counter_cur_thread);
+   #endif
+   return x;
+ }
 
 }
 
@@ -467,9 +546,14 @@ return end_mcrypt(td);
 }
 
 
-int init_mcrypt_cbc(const int algorithm, void * key,const int lenofkey)
+int init_mcrypt_cbc(const int algorithm, void * key, int lenofkey)
 {
 
+int x;
+
+#ifdef HAVE_LIBPTHREAD
+pthread_mutex_lock(&counter_cur_thread);
+#endif
 cur_thread++;
 
 if (cur_thread < MAX_THREADS && finished_threads==0) {
@@ -487,9 +571,15 @@ else {
   while (keyword_given[cur_thread]!=NULL && cur_thread <= MAX_THREADS) {
    cur_thread++;
   }  
-  if (cur_thread >= MAX_THREADS) return -1;
+  if (cur_thread >= MAX_THREADS) {
+    #ifdef HAVE_LIBPTHREAD
+    pthread_mutex_unlock(&counter_cur_thread);
+    #endif
+    return (-1);
+  }
 }
 
+
 /* For cbc */
   previous_ciphertext[cur_thread]=calloc(1,get_block_size(algorithm));
   previous_cipher[cur_thread]=malloc(get_block_size(algorithm));
@@ -497,7 +587,21 @@ else {
 
 /* End cbc */
 
-return init_mcrypt(algorithm, key, lenofkey);
+x=init_mcrypt(algorithm, key, lenofkey);
+
+ if (x<0) {
+   end_mcrypt_cbc(cur_thread);
+   #ifdef HAVE_LIBPTHREAD
+   pthread_mutex_unlock(&counter_cur_thread);
+   #endif
+   return -1;
+ }
+ else {
+   #ifdef HAVE_LIBPTHREAD
+   pthread_mutex_unlock(&counter_cur_thread);
+   #endif
+   return x;
+ }
 
 }
 
@@ -568,11 +672,16 @@ return end_mcrypt(td);
 }
 
 
-int init_mcrypt_cfb(const int algorithm, void * key,const int lenofkey, void *IV)
+int init_mcrypt_cfb(const int algorithm, void * key, int lenofkey, void *IV)
 {
+int x;
 
+#ifdef HAVE_LIBPTHREAD
+pthread_mutex_lock(&counter_cur_thread);
+#endif
 cur_thread++;
 
+
 if (cur_thread < MAX_THREADS && finished_threads==0) {
 
 /* for cfb */
@@ -588,9 +697,15 @@ else {
   while (keyword_given[cur_thread]!=NULL && cur_thread <= MAX_THREADS) {
    cur_thread++;
   }  
-  if (cur_thread >= MAX_THREADS) return -1;
+  if (cur_thread >= MAX_THREADS) {
+    #ifdef HAVE_LIBPTHREAD
+    pthread_mutex_unlock(&counter_cur_thread);
+    #endif
+    return -1;
+  }
 }
 
+
 /* For cfb */
   s_register[cur_thread]=malloc( get_block_size(algorithm));
   sd_register[cur_thread]=malloc( get_block_size(algorithm));
@@ -601,8 +716,21 @@ else {
 
 /* End cfb */
 
-return init_mcrypt(algorithm, key, lenofkey);
+x=init_mcrypt( algorithm, key, lenofkey);
 
+ if (x<0) {
+   end_mcrypt_cfb(cur_thread);
+   #ifdef HAVE_LIBPTHREAD
+   pthread_mutex_unlock(&counter_cur_thread);
+   #endif
+   return -1;
+ }
+ else {
+   #ifdef HAVE_LIBPTHREAD
+   pthread_mutex_unlock(&counter_cur_thread);
+   #endif
+   return x;
+ }
 }
 
 int mcrypt_cfb(int td, void *plaintext,int len) /* plaintext is 1 byte (8bit cfb) */
@@ -655,3 +783,54 @@ int mdecrypt_cfb(int td, void *plaintext,int len) /* plaintext is 1 byte (8bit c
 
   return 0;
 }
+
+
+char* get_algorithms_name(int algorithm) {
+
+char *x=malloc(12);
+
+  switch(algorithm) {
+   case DES:
+    strcpy(x,"DES");
+    return x;
+    break;
+   case TripleDES:
+    strcpy(x,"3DES");
+    return x;
+    break;
+   case ThreeWAY:
+    strcpy(x,"3-WAY");
+    return x;
+    break;
+   case BLOWFISH:
+    strcpy(x,"BLOWFISH");
+    return x;
+    break;
+   case TWOFISH:
+    strcpy(x,"TWOFISH");
+    return x;
+    break;    
+   case GOST:
+    strcpy(x,"GOST");
+    return x;
+    break;
+   case SAFER64:
+    strcpy(x,"SAFER-SK64");
+    return x;
+    break;
+   case SAFER128:
+    strcpy(x,"SAFER-SK128");
+    return x;
+    break;
+   case CAST128:
+    strcpy(x,"CAST-128");
+    return x;
+    break;
+   case TEAN:
+    strcpy(x,"xTEA");
+    return x;
+    break;
+  }
+return NULL;
+
+}
