@@ -40,3 +40,6 @@
 #  error "Neither memmove nor bcopy exists on your system."
 # endif
 #endif
+
+#define ENCRYPT 0
+#define DECRYPT 1
