@@ -53,7 +53,7 @@ char *btmp;
  return btmp;
 }
 
-char * get_password(int numofchar,int bitmode,int mode,int type)
+char * get_password(int numofchar,int bitmode,int mode,int type, unsigned int* len)
 {
 int x=0;
 char *tmp;
@@ -81,7 +81,8 @@ char msg[100];
 
    if ((strlen(tmp) < (numofchar*8)/7) && (type!=BLOWFISH)) { fprintf(stderr,_("Warning: keyword is too small for this algorithm\n")); }
    if (strlen(tmp) > ((numofchar*8)/7)+x) { fprintf(stderr,_("Warning: Reducing the size of keyword\n")); }
-   strip8bit(tmp,strlen(tmp));
+
+   *len=strip8bit(tmp,strlen(tmp));
 }
 
 return tmp;
@@ -91,36 +92,41 @@ return tmp;
 
 int show_mode(int ende, int stype,int smode,char *output,int bitmode)
 {
-  
-  if (stype==DES) fprintf (stderr,_( "Algorithm: DES \n"));  
-  if (stype==TripleDES) fprintf (stderr,_( "Algorithm: 3DES\n"));
-  if (stype==ThreeWAY) fprintf (stderr,_( "Algorithm: 3-WAY\n"));
-  if (stype==BLOWFISH) fprintf (stderr,_( "Algorithm: Blowfish\n"));
-  if (stype==GOST) fprintf (stderr,_( "Algorithm: GOST\n"));
-  if (stype==SAFER64) fprintf (stderr,_( "Algorithm: SAFER-SK64\n"));  
-  if (stype==SAFER128) fprintf (stderr,_( "Algorithm: SAFER-SK128\n"));
-  if (stype==CAST128) fprintf (stderr,_( "Algorithm: CAST-128\n"));
-  if (stype==TEAN) fprintf (stderr,_( "Algorithm: TEA (extended)\n"));  
-  if (stype==CRYPT) {
-    fprintf (stderr,_( "Algorithm: Unix Crypt\n"));  
-  }
-  else {
+char *x=get_algorithms_name(stype);
 
-  if (smode==CBC) fprintf (stderr,_( "Mode: CBC\n"));
-  if (smode==ECB) fprintf (stderr,_( "Mode: ECB\n"));
-  if (smode==CFB) fprintf (stderr,_( "Mode: CFB\n"));
-  
+
+ fprintf (stderr,_( "Algorithm: %s\n"),x);
+ free(x); 
+   
+ if (stype!=CRYPT) {
+   switch(smode) {
+    case CBC:
+     fprintf (stderr,_( "Mode: CBC\n"));
+     break;
+    case ECB:
+     fprintf (stderr,_( "Mode: ECB\n"));
+     break;
+    case CFB:
+     fprintf (stderr,_( "Mode: CFB\n"));
+     break;
+   }
+ }
+
+  switch (bitmode) {
+   case BITS7:
+    fprintf (stderr,_( "Keyword Bit Mode: 7bits/char\n"));
+    break;
+   case BITS8:
+    fprintf (stderr,_( "Keyword Bit Mode: 8bits/char\n"));  
+    break;
   }
-  
-  if (bitmode==BITS7) fprintf (stderr,_( "Keyword Bit Mode: 7bits/char\n"));
-  if (bitmode==BITS8) fprintf (stderr,_( "Keyword Bit Mode: 8bits/char\n"));  
-    
+
    if (ende==ENCRYPT) {
      fprintf (stderr,_( "Crypting to %s ...\n"),output);
-     }
+   }
    else {
      fprintf (stderr, _("Decrypting to %s ...\n"),output);
-     }
+   }
   
 return 0;  
 }
@@ -154,7 +160,7 @@ char buf[6];
  }
  else {
   fstream=fopen(filename,"r");
-  read_lock( fileno((FILE*)fstream));
+  if (read_lock( fileno((FILE*)fstream))==-1) return (-1);
  } 
 
  fread(buf, 1, 6, fstream);
@@ -274,3 +280,26 @@ if (stream_flag==FALSE) remove(outfile); /* Delete the file we were writing to *
 
 }
 
+
+char** read_key_file(char * file, int* num) {
+
+  FILE* FROMF;
+  char keyword[127], **keys=NULL;
+  int x=0;
+
+  FROMF=fopen(file,"r");
+  if (FROMF==NULL) {perror("fopen");exit(1);}
+  if (read_lock( fileno((FILE*)FROMF))==-1) return 0;
+
+  while (fgets( keyword, 127, FROMF)!=NULL) {
+    x++;
+
+    keys=realloc(keys, x*sizeof(char*));
+    keys[x-1]=malloc(strlen(keyword)+1);
+    strcpy( keys[x-1], keyword);
+
+  }
+*num=x;
+return keys;
+
+}
