@@ -1,5 +1,6 @@
-char * get_password(int numofchar,int bitmode,int mode,int type);
+char * get_password(int numofchar,int bitmode,int mode,int type, unsigned int *len);
 int show_mode(int ende, int stype,int smode,char *output,int bitmode);
+char** read_key_file(char * file, int* num);
 
 char *my_getpass();
 
