@@ -32,7 +32,7 @@
 #include <sunlink.h>
 #include <crc32.h>
 #include <locks.h>
-#include <crypt.h>
+#include <enigma.h>
 
  unsigned int bit7_flag=BITS7;
  unsigned int stream_flag=FALSE;
@@ -47,18 +47,20 @@
 void usage(char *name)
 {
 
-     fprintf(stderr,_("Usage: mcrypt FILE1 FILE2 ...\n"));
-     fprintf(stderr,_("Usage: mcrypt -d FILE1 FILE2 ...\n\n"));
+     fprintf(stderr,_("Usage: mcrypt [-dLFu87bhv] [-f keyfile] [-k keyword] [-m mode] [-a algorithm] [file ...]\n\n"));
 
-     fprintf(stderr,_("mini-crypt encrypts/decrypts files using several symmetric block algorithms.\n\
+     fprintf(stderr,_("mini-crypt encrypts/decrypts files using several symmetric block algorithms.\n"));
+     fprintf(stderr,_("Legal parameters are:\n\
 \n\
  -d --decrypt                 decrypts\n\
  -a --algorithm=ALGORITHM     specify the encryption and decryption algorithm.\n\
                               Legal parameters are: 3way, des, 3des, gost, tea,\n\
-                              safer64, safer128, cast128, crypt and blowfish. \n\
-                              (Default is blowfish)\n\
- -m --mode=MODE               specify the mode (cfb, ecb or cbc). (Default is cbc)\n\
+                              safer64, safer128, cast128, crypt, twofish\n\
+                              and blowfish. (Default is blowfish)\n\
+ -m --mode=MODE               specify the mode (cfb, ecb or cbc).\n\
+                              (Default is cbc)\n\
  -k --key=KEYWORD             keyword to use.\n\
+ -f --file=FILENAME           file to read the keyword from.\n\
  -7 --7bit                    keyword in 7bit mode. (default)\n\
  -8 --8bit                    keyword in 8bit mode.\n\
  -u --unlink                  unlink (and destroy) the input file after\n\
@@ -66,8 +68,9 @@ void usage(char *name)
  -L --license                 displays license information.\n\
  -q --quiet                   suppress some non critical warnings.\n\
  -V --verbose                 some more information is displayed.\n\
- -b --bare                    Do not keep algorithm information in the encrypted file.\n\
- -f --force                   forces output to stdout.\n\
+ -b --bare                    Do not keep algorithm information in the encrypted\n\
+                              file.\n\
+ -F --force                   forces output to stdout.\n\
  \n\
  -h --help                    prints this help\n\
  -v --version                 prints the version number\n\
@@ -83,9 +86,9 @@ int main(int argc,char **argv)
 {
 	short int ch,ein=0,din=0,kin=0,force=0,return_val=0;
 	char *einfile=0 ,*dinfile=0, *keyword=0,*modet=0,*typet=0;
-	char **file; 
+	char **file, **keyfile=NULL;
 	extern char *optarg;
-	int x,y,i,file_count=0;
+	int x,y,i,file_count=0, keys=0;
 
 
 struct option longopts[] = {
@@ -96,7 +99,8 @@ struct option longopts[] = {
 	    { "mode", 1, 0, 'm' },
 	    { "algorithm", 1, 0, 'a' },
 	    { "key", 1, 0, 'k' },
-	    { "force", 0, 0, 'f' },
+	    { "file", 1, 0, 'f' },
+	    { "force", 0, 0, 'F' },
 	    { "7bit", 0, 0, '7' },	    
 	    { "8bit", 0, 0, '8' },
 	    { "quiet", 0, 0, 'q'},
@@ -124,6 +128,9 @@ struct option longopts[] = {
   textdomain (PACKAGE);
 #endif
 
+/* For RAND32... Called only here */
+ srand(time(0));
+
 /* Examine how we were called */
    if (strstr(argv[0],"decrypt")!=NULL) {
      strcpy( argv[0], "mdecrypt");
@@ -141,7 +148,7 @@ struct option longopts[] = {
 
         file=malloc((argc-optind)*sizeof(char*)); 
 
-	while( (ch=getopt_long(argc,argv,"f78bLVuqda:m:k:hv",longopts, (int *)0 )) != EOF )
+	while( (ch=getopt_long(argc,argv,"F78bLVuqda:m:k:f:hv",longopts, (int *)0 )) != EOF )
 
 		switch(ch) {
 	
@@ -175,7 +182,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 			ein=FALSE;
 			break;
 
-		case 'f':
+		case 'F':
 			force=TRUE;
 			break;
 
@@ -203,11 +210,26 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 			bit7_flag=BITS8;
 			break;			
 
+		case 'f':
+                        if (kin==1) {
+                          fprintf(stderr, _("--key and --file parameters are mutually exclusive.\n"));
+                          exit(1);
+                        }
+			kin=2;
+			keyfile = read_key_file(optarg, &keys);
+			if (keys==0) kin=0;
+			break;
+
 		case 'k':
+                        if (kin==2) {
+                          fprintf(stderr, _("--key and --file parameters are mutually exclusive.\n"));
+                          exit(1);
+                        }
 			kin=1;
 			keyword= malloc(strlen(optarg)+10);
 			strcpy(keyword,optarg);
-			break;
+			break;			
+
 		case 'm':
 			modet= malloc(strlen(optarg)+1);
 			strcpy(modet,optarg);
@@ -221,7 +243,10 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 			  else {
                             if (strncasecmp(modet,"CFB",3)==0) {
                               mode=CFB;
-	  	            }			  
+	  	            }
+	  	            else {
+                              mode=CBC; /* Default */
+	  	            }
 			  }
                         
                         }
@@ -267,7 +292,15 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
                                     }
                                     else {
                                        if (strncasecmp(typet,"TEAN",4)==0 || strncasecmp(typet,"TEA",3)==0) {
-                                       type=TEAN;
+                                        type=TEAN;
+                                       }
+                                       else { 
+                                          if (strncasecmp(typet,"TWOFISH",7)==0) {
+                                           type=TWOFISH;
+                                          }
+                                          else { /* Default */
+                                           type=BLOWFISH;
+                                          }
                                        }
                                     }
                                   }
@@ -317,11 +350,21 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
     if (stream_flag==TRUE) file_count++;
 
 
+
 /* Do as many times as many files we got */
 for(i=0;i<file_count;i++) {
 
     if(i!=0) { if (outfile!=NULL) free(outfile); }
 
+/* If keyword file specified choose the i keyword */
+    if (kin==2 && i<=(keys-1)) {
+       keyword=keyfile[i];
+       if (i!=0) { 
+         Bzero(keyfile[i-1],strlen(keyfile[i-1])); 
+         free(keyfile[i-1]); /* Free previous keyword */
+       }
+    }
+
 #ifdef HAVE_STAT
     if ( is_normal_file( file[i]) == FALSE && stream_flag==FALSE ) {
       fprintf( stderr, _("%s: %s is not a regular file. Skipping...\n"), argv[0], file[i]);
@@ -404,9 +447,16 @@ for(i=0;i<file_count;i++) {
 	if (din==TRUE) {
 
            if (bare_flag==FALSE) {
-             if (check_file_head(file[i],&type,&mode, &bit7_flag, stream_flag)!=FALSE) {
-               fprintf(stderr,_("%s: no headers. Assuming encrypted file with Unix crypt.\n"),argv[0]); 
-               type=CRYPT;
+             x=check_file_head(file[i],&type,&mode, &bit7_flag, stream_flag);
+             if (x!=FALSE) {
+               if (x==-1) {
+                 type=NO_TYPE; /* Lock failed */
+               }
+               else { /* Unix Crypt */
+                 fprintf(stderr,_("%s: no headers. Assuming encrypted file with Unix crypt.\n"),argv[0]); 
+                 fflush(stderr);
+                 type=CRYPT;
+               }
              }
            }
 
@@ -420,6 +470,7 @@ for(i=0;i<file_count;i++) {
 			        case TripleDES:
 				case ThreeWAY:
 				case BLOWFISH:
+                                case TWOFISH:
 				case GOST:
 				case SAFER64:
 				case SAFER128:
@@ -430,7 +481,10 @@ for(i=0;i<file_count;i++) {
 				case CRYPT:
 				 x=encrypt_ucrypt(dinfile,outfile,keyword);
 				 break;			 
-                        	default:
+                                case NO_TYPE:
+                                 x=1;
+                                 break;
+                                default:
 	                         fprintf(stderr,_("Algorithm unsupported in this version\n"));
         	                 x=1;
                 	         break;
@@ -484,6 +538,7 @@ for(i=0;i<file_count;i++) {
 		        case TripleDES:
 	        	case ThreeWAY:
 	        	case BLOWFISH:
+                        case TWOFISH:
 	        	case GOST:
 	        	case SAFER64:
 	        	case SAFER128:
@@ -553,6 +608,7 @@ int encrypt_ucrypt(char *fromfile,char *tofile,char *key)
 {
 word32 plaintext[2]; /* 64 bit */
 int ch=0,how=8; /* how needs to be initialized to 8 */
+unsigned int len;
 char keyword[13]; 
 char *tmp;
 FILE *TOF;
@@ -562,7 +618,7 @@ FILE *FROMF;
  * It's purpose is to be compatible with the unix crypt not safe.
  * It isn't a block algorithm after all.
  */
-mode=2;
+
 bit7_flag=BITS8;
 
 
@@ -575,20 +631,20 @@ else {
 
 	FROMF=fopen(fromfile,"rb");
 	if (FROMF==NULL) {perror("fopen");return 1;}
-	read_lock( fileno((FILE*)FROMF));
+	if (read_lock( fileno((FILE*)FROMF))==-1) return 1;
 	
 	TOF=fopen(tofile,"wb");
 	if (TOF==NULL) {perror("fopen");return 1;}
-	write_lock( fileno((FILE*)TOF));
+	if (write_lock( fileno((FILE*)TOF))==-1) return 1;
 }
 
         Bzero(keyword,sizeof(keyword));
 
 	if (key==NULL) 	{
           if (quiet==TRUE && stream_flag==FALSE) fprintf(stderr, _("File: %s\n"),fromfile);
-          tmp=get_password(8,bit7_flag,DECRYPT,type);
+          tmp=get_password(8, bit7_flag, DECRYPT, type, &len);
           if (tmp==NULL) return 1;
-          memmove(keyword, tmp, strlen(tmp));
+          memmove(keyword, tmp, len);
           Bzero(tmp, strlen(tmp));
 
 	}
@@ -681,7 +737,7 @@ FILE * TOF;
 if (mode==CBC || mode==CFB) {
  IV=malloc(blocksize);
  /* Initialisation Vector */
- srand(time(0));
+
 
 /* Random number generator */
 
@@ -698,11 +754,11 @@ if (mode==CBC || mode==CFB) {
   else {
 	FROMF=fopen(fromfile,"rb");
 	if (FROMF==NULL) {perror("fopen");return 1;}
-        read_lock( fileno((FILE*)FROMF));
+        if (read_lock( fileno((FILE*)FROMF))==-1) return 1;
         
 	RTOF=fopen(tofile,"wb");
 	if (RTOF==NULL) {perror("fopen");return 1;}
-        write_lock( fileno((FILE*)RTOF));
+        if (write_lock( fileno((FILE*)RTOF))==-1) return 1;
         
   }
 
@@ -717,9 +773,8 @@ if (mode==CBC || mode==CFB) {
 
 	if (key==NULL) 	{
           if (quiet==TRUE && stream_flag==FALSE) fprintf(stderr, _("File: %s\n"),fromfile);
-          tmp=get_password( keysize, bit7_flag, ENCRYPT, algorithm );
+          tmp=get_password( keysize, bit7_flag, ENCRYPT, algorithm , &len);
           if (tmp==NULL) return 1;
-          len=strlen(tmp);
 
           if (len > keysize) len=keysize;
           memmove(keyword, tmp, len);
@@ -740,8 +795,7 @@ if (mode==CBC || mode==CFB) {
            else { /* 7bit mode */
                len=strlen(key);
                memmove(tmpkey, key, len);
-               strip8bit(tmpkey, len);
-               len=strlen(tmpkey);
+               len=strip8bit(tmpkey, len);
                if (len > keysize) {
                   fprintf(stderr,_("Warning: Reducing the size of the key\n"));
                   len=keysize;
@@ -754,11 +808,6 @@ if (mode==CBC || mode==CFB) {
  if (quiet==FALSE) show_mode(ENCRYPT,algorithm,mode,tofile,bit7_flag);
 
 
-//  if (algorithm != BLOWFISH) {
-//    len=keysize; /* Only Blowfish has non-fixed key */
-//  }
-
-
   switch (mode) {
    case ECB:
     td = init_mcrypt_ecb(algorithm,keyword,len);
@@ -774,6 +823,11 @@ if (mode==CBC || mode==CFB) {
     break;
   }  
 
+   if (td<0) { 
+     fprintf (stderr, _("Unknown algorithm error\n"));
+     return 1;
+   }
+
  if (bare_flag==FALSE) clear_crc32();
 
 
@@ -890,6 +944,9 @@ switch (mode) {
   while(ch==0);
   break;
 
+  default:
+   return 1;
+   break;
 } /*  switch */
 
  if (bare_flag==FALSE) {
@@ -902,11 +959,11 @@ switch (mode) {
 #endif   
 
 
-
 /* encrypt the crc32, but first put some random data there 
  */
-   for(j=0; j<blocksize/sizeof(word32); j++)
+   for(j=0; j<blocksize/sizeof(word32); j++) {
      ciphertext[j] = RAND32;
+   }
    memmove(ciphertext,&fcrc32,sizeof(fcrc32));
    mcrypt(td, ciphertext);
    fseek(TOF,6,SEEK_SET);
@@ -915,8 +972,8 @@ switch (mode) {
  }
 
 /* Copy the temporary to RTOF */
-   fseek(TOF,0,SEEK_SET);
 
+ fseek(TOF,0,SEEK_SET);
  copychar=malloc(4096);
 
  do {
@@ -925,6 +982,7 @@ switch (mode) {
  } while ( how!=0 );
 
  free(copychar);
+
 /* End of copy */
    
 /* close and unlock files*/
@@ -1005,11 +1063,11 @@ else {
 
 	FROMF=fopen(fromfile,"rb");
 	if (FROMF==NULL) {perror("fopen");return 1;}
-        read_lock( fileno((FILE*)FROMF));
+        if (read_lock( fileno((FILE*)FROMF))==-1) return 1;
         
 	RTOF=fopen(tofile,"wb");
 	if (RTOF==NULL) {perror("fopen");return 1;}
-        write_lock( fileno((FILE*)RTOF));
+        if (write_lock( fileno((FILE*)RTOF))==-1) return 1;
 
         if (bare_flag==FALSE) fseek( FROMF, 6, SEEK_CUR);
         /* In streams this has been done when checking for headers. */
@@ -1019,9 +1077,8 @@ else {
   
 	if (key==NULL) {
           if (quiet==TRUE && stream_flag==FALSE) fprintf(stderr, _("File: %s\n"),fromfile);
-          tmp=get_password(keysize,bit7_flag,DECRYPT,algorithm);
+          tmp=get_password(keysize,bit7_flag,DECRYPT,algorithm, &len);
           if (tmp==NULL) return 1;
-          len=strlen(tmp);
                               
           if (len>keysize) len=keysize;
           memmove(keyword, tmp, len);
@@ -1040,8 +1097,7 @@ else {
             else { /* 7bit mode */
                len=strlen(key);
                memmove(tmpkey, key, len);
-               strip8bit( tmpkey, len);
-               len=strlen(tmpkey);
+               len=strip8bit( tmpkey, len);
                if (len > keysize) {
                   fprintf(stderr,_("Warning: Reducing the size of the key\n"));
                   len=keysize;
@@ -1060,10 +1116,10 @@ if (quiet==FALSE) show_mode(DECRYPT,algorithm,mode,tofile,bit7_flag);
       td = init_mcrypt_cbc(algorithm,keyword,len);
       break;
     case ECB:
-      td = init_mcrypt_ecb(algorithm,keyword,len);
+      td = init_mcrypt_ecb( algorithm,keyword,len);
       break;
     case CFB:
-      td = init_mcrypt_ecb(algorithm,keyword,len);
+      td = init_mcrypt_ecb( algorithm,keyword,len);
       break;      
     default:
       return 1;
@@ -1245,6 +1301,11 @@ switch (mode) {
    }
    while(ch==0);
    break;   
+ 
+ default:
+   return 1;
+   break;
+
 } /* switch */
 
 
