@@ -61,7 +61,7 @@ FILE *fstream;
 # endif      
   }
 
-  srand(time(0));
+/*  srand(time(0)); */
   
   for (j=0;j<5;j++) {
        if (fseek(fstream,0,SEEK_SET)==-1) {perror("fseek"); return 1;}
